#!/usr/bin/env python3

# 把 cbp 转换为一个 Makefile 文件

import argparse
import sys
import os
import xml.etree.ElementTree as ET

COMPILERS = {
  "dv10": {
    "cc": "C:/JL/dv10/bin/dv10-elf-gcc.exe",
    "cxx": "C:/JL/dv10/bin/dv10-elf-g++.exe",
    "ld": "C:/JL/dv10/bin/dv10-elf-ld.exe",
    "ar": "C:/JL/dv10/bin/dv10-elf-ar.exe",
    "cc_opts": [],
    "ld_opts": [],
    "inc_dirs": [
      "C:/JL/dv10/include"
    ],
    "lib_dirs": [
      "C:/JL/dv10/lib"
    ],
    "libs": [
      "C:/JL/dv10/lib/libc.a",
      "C:/JL/dv10/lib/libgcc.a",
      "C:/JL/dv10/lib/libgcov.a",
      "C:/JL/dv10/lib/libiberty.a",
      "C:/JL/dv10/lib/libm.a"
    ]
  },
  "dv12": {
    "cc": "C:/JL/dv12/bin/dv12-elf-gcc.exe",
    "cxx": "C:/JL/dv12/bin/dv12-elf-g++.exe",
    "ld": "C:/JL/dv12/bin/dv12-elf-ld.exe",
    "ar": "C:/JL/dv12/bin/dv12-elf-ar.exe",
    "cc_opts": [],
    "ld_opts": [],
    "inc_dirs": [
      "C:/JL/dv12/include"
    ],
    "lib_dirs": [
      "C:/JL/dv12/lib"
    ],
    "libs": [
      "C:/JL/dv12/lib/libc.a",
      "C:/JL/dv12/lib/libgcc.a",
      "C:/JL/dv12/lib/libgcov.a",
      "C:/JL/dv12/lib/libiberty.a",
      "C:/JL/dv12/lib/libm.a"
    ]
  },
  "pi32_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/lto-ar.exe",
    "cc_opts": [
      "-target pi32",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -pi32-memreg-opt",
      "-mllvm -pi32-mem-offset-adj-opt",
      "-mllvm -pi32-const-spill",
      "-mllvm -pi32-enable-jz",
      "-mllvm -pi32-tailcall-opt",
      "-mllvm -inline-threshold=5",
      "-mllvm -pi32-enable-itblock=1",
      "-Oz"
    ],
    "ld_opts": [
      "--plugin-opt=-pi32-memreg-opt",
      "--plugin-opt=-pi32-mem-offset-adj-opt",
      "--plugin-opt=-pi32-const-spill",
      "--plugin-opt=-pi32-enable-jz",
      "--plugin-opt=-pi32-tailcall-opt",
      "--plugin-opt=-inline-threshold=5",
      "--plugin-opt=-pi32-enable-itblock=1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/include/libc"
    ],
    "lib_dirs": [
      "C:/JL/pi32/libc"
    ],
    "libs": [
      "C:/JL/pi32/lib/libc.a",
      "C:/JL/pi32/lib/libm.a",
      "C:/JL/pi32/lib/libcompiler-rt.a"
    ]
  },
  "pi32_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/lto-ar.exe",
    "cc_opts": [
      "-target pi32",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -pi32-memreg-opt",
      "-mllvm -pi32-mem-offset-adj-opt",
      "-mllvm -pi32-const-spill",
      "-mllvm -pi32-enable-jz",
      "-mllvm -pi32-tailcall-opt",
      "-mllvm -inline-threshold=5",
      "-mllvm -pi32-enable-itblock=1",
      "-Oz",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=-pi32-memreg-opt",
      "--plugin-opt=-pi32-mem-offset-adj-opt",
      "--plugin-opt=-pi32-const-spill",
      "--plugin-opt=-pi32-enable-jz",
      "--plugin-opt=-pi32-tailcall-opt",
      "--plugin-opt=-inline-threshold=5",
      "--plugin-opt=-pi32-enable-itblock=1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/include/libc"
    ],
    "lib_dirs": [
      "C:/JL/pi32/libc"
    ],
    "libs": [
      "C:/JL/pi32/lib/libc.a",
      "C:/JL/pi32/lib/libm.a",
      "C:/JL/pi32/lib/libcompiler-rt.a"
    ]
  },
  "pi32_lto_compiler_without_libc": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/lto-ar.exe",
    "cc_opts": [
      "-target pi32",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -pi32-memreg-opt",
      "-mllvm -pi32-mem-offset-adj-opt",
      "-mllvm -pi32-const-spill",
      "-mllvm -pi32-enable-jz",
      "-mllvm -pi32-tailcall-opt",
      "-mllvm -inline-threshold=5",
      "-mllvm -pi32-enable-itblock=1",
      "-Oz",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=-pi32-memreg-opt",
      "--plugin-opt=-pi32-mem-offset-adj-opt",
      "--plugin-opt=-pi32-const-spill",
      "--plugin-opt=-pi32-enable-jz",
      "--plugin-opt=-pi32-tailcall-opt",
      "--plugin-opt=-inline-threshold=5",
      "--plugin-opt=-pi32-enable-itblock=1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/include/libc"
    ],
    "lib_dirs": [
      "C:/JL/pi32/libc"
    ],
    "libs": [
      "C:/JL/pi32/lib/libcompiler-rt.a"
    ]
  },
  "pi32_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/lto-ar.exe",
    "cc_opts": [
      "-target pi32",
      "-integrated-as"
    ],
    "ld_opts": [],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": []
  },
  "pi32v2_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -inline-threshold=5",
      "-Oz"
    ],
    "ld_opts": [
      "--plugin-opt=-inline-threshold=5"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/libc.a",
      "C:/JL/pi32/pi32v2-lib/libm.a",
      "C:/JL/pi32/pi32v2-lib/libcompiler-rt.a"
    ]
  },
  "pi32v2_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -inline-threshold=5",
      "-Oz",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=-inline-threshold=5"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/libc.a",
      "C:/JL/pi32/pi32v2-lib/libm.a",
      "C:/JL/pi32/pi32v2-lib/libcompiler-rt.a"
    ]
  },
  "pi32v2_lto_compiler_without_libc": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-flto",
      "-target pi32v2",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -inline-threshold=5",
      "-Oz"
    ],
    "ld_opts": [
      "-flto",
      "--plugin-opt=-inline-threshold=5"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/libcompiler-rt.a"
    ]
  },
  "pi32v2_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as"
    ],
    "ld_opts": [],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": []
  },
  "pi32v2_r1_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as",
      "-mcpu=r1"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r1"
    ],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": []
  },
  "pi32v2_r1_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r1",
      "-integrated-as",
      "-mllvm -inline-threshold=5",
      "-Oz",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=-inline-threshold=5",
      "--plugin-opt=mcpu=r1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r1"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r1/libc.a",
      "C:/JL/pi32/pi32v2-lib/r1/libm.a",
      "C:/JL/pi32/pi32v2-lib/r1/libcompiler-rt.a"
    ]
  },
  "pi32v2_r1_large_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r1",
      "-integrated-as",
      "-mllvm -inline-threshold=5",
      "-Oz",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=-inline-threshold=5",
      "--plugin-opt=mcpu=r1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r1-large"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r1-large/libc.a",
      "C:/JL/pi32/pi32v2-lib/r1-large/libm.a",
      "C:/JL/pi32/pi32v2-lib/r1-large/libcompiler-rt.a"
    ]
  },
  "pi32v2_r2_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as",
      "-mcpu=r2"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r2"
    ],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": []
  },
  "pi32v2_r2_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r2",
      "-integrated-as",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r2"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r2"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r2/libc.a",
      "C:/JL/pi32/pi32v2-lib/r2/libm.a",
      "C:/JL/pi32/pi32v2-lib/r2/libcompiler-rt.a"
    ]
  },
  "pi32v2_r3_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as",
      "-mcpu=r3",
      "-mfprev1"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r3",
      "--plugin-opt=-mattr=+fprev1"
    ],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": []
  },
  "pi32v2_r3_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r3",
      "-integrated-as",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r3",
      "--plugin-opt=-mattr=+fprev1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r3"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r3/libc.a",
      "C:/JL/pi32/pi32v2-lib/r3/libm.a",
      "C:/JL/pi32/pi32v2-lib/r3/libcompiler-rt.a"
    ]
  },
  "pi32v2_r3_large_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r3",
      "-integrated-as",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r3",
      "--plugin-opt=-mattr=+fprev1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r3-large"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r3-large/libc.a",
      "C:/JL/pi32/pi32v2-lib/r3-large/libm.a",
      "C:/JL/pi32/pi32v2-lib/r3-large/libcompiler-rt.a"
    ]
  },
  "pi32v2_r5_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as",
      "-mcpu=r5",
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r5",
    ],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": []
  },
  "pi32v2_r5_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r5",
      "-integrated-as",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r5",
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r5"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r5/libc.a",
      "C:/JL/pi32/pi32v2-lib/r5/libm.a",
      "C:/JL/pi32/pi32v2-lib/r5/libcompiler-rt.a"
    ]
  },
  "pi32v2_r5_large_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r5",
      "-integrated-as",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r5",
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r5-large"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r5-large/libc.a",
      "C:/JL/pi32/pi32v2-lib/r5-large/libm.a",
      "C:/JL/pi32/pi32v2-lib/r5-large/libcompiler-rt.a"
    ]
  },
  "q32s_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/q32s-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/q32s-lto-ar.exe",
    "cc_opts": [
      "-target q32s",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -inline-threshold=5",
      "-Oz"
    ],
    "ld_opts": [
      "--plugin-opt=-inline-threshold=5"
    ],
    "inc_dirs": [
      "C:/JL/pi32/q32s-include"
    ],
    "lib_dirs": [
      "C:/JL/pi32/q32s-lib"
    ],
    "libs": [
      "C:/JL/pi32/q32s-lib/libc.a",
      "C:/JL/pi32/q32s-lib/libm.a",
      "C:/JL/pi32/q32s-lib/libcompiler-rt.a"
    ]
  },
  "q32s_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/q32s-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/q32s-lto-ar.exe",
    "cc_opts": [
      "-flto",
      "-target q32s",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -inline-threshold=5",
      "-Oz"
    ],
    "ld_opts": [
      "-flto",
      "--plugin-opt=-inline-threshold=5"
    ],
    "inc_dirs": [
      "C:/JL/pi32/q32s-include"
    ],
    "lib_dirs": [
      "C:/JL/pi32/q32s-lib"
    ],
    "libs": [
      "C:/JL/pi32/q32s-lib/libc.a",
      "C:/JL/pi32/q32s-lib/libm.a",
      "C:/JL/pi32/q32s-lib/libcompiler-rt.a"
    ]
  },
  "q32s_lto_compiler_without_libc": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/q32s-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/q32s-lto-ar.exe",
    "cc_opts": [
      "-flto",
      "-target q32s",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -inline-threshold=5",
      "-Oz"
    ],
    "ld_opts": [
      "-flto",
      "--plugin-opt=-inline-threshold=5"
    ],
    "inc_dirs": [
      "C:/JL/pi32/q32s-include"
    ],
    "lib_dirs": [
      "C:/JL/pi32/q32s-lib"
    ],
    "libs": [
      "C:/JL/pi32/q32s-lib/libcompiler-rt.a"
    ]
  },
  "q32s_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/q32s-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/q32s-lto-ar.exe",
    "cc_opts": [
      "-target q32s",
      "-integrated-as"
    ],
    "ld_opts": [],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": []
  },
  "f95_assembler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/f95-ld.exe",
    "ar": "C:/JL/pi32/bin/f95-ar.exe",
    "cc_opts": [
      "-target f95",
      "-integrated-as"
    ],
    "ld_opts": [],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": []
  },
  "f96_assembler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/f95-ld.exe",
    "ar": "C:/JL/pi32/bin/f95-ar.exe",
    "cc_opts": [
      "-target f95",
      "-integrated-as",
      "-mcpu=r1"
    ],
    "ld_opts": [],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": []
  },
  "ac46_compiler": {
    "cc": "C:/JL/ac46/bin/ac46-elf-gcc.exe",
    "cxx": "C:/JL/ac46/bin/ac46-elf-g++.exe",
    "ld": "C:/JL/ac46/bin/ac46-elf-ld.exe",
    "ar": "C:/JL/ac46/bin/ac46-elf-ar.exe",
    "cc_opts": [],
    "ld_opts": [],
    "inc_dirs": [
      "C:/JL/ac46/ac46-elf/include"
    ],
    "lib_dirs": [
      "C:/JL/ac46/ac46-elf/lib"
    ],
    "libs": [
      "C:/JL/ac46/ac46-elf/lib/libc.a",
      "C:/JL/ac46/ac46-elf/lib/libgcc.a",
      "C:/JL/ac46/ac46-elf/lib/libgcov.a",
      "C:/JL/ac46/ac46-elf/lib/libm.a"
    ]
  },
  "mc": {
    "cc": "C:/JL/mc/bin/mc-uclinux-gcc.exe",
    "cxx": "C:/JL/mc/bin/mc-uclinux-g++.exe",
    "ld": "C:/JL/mc/bin/mc-uclinux-ld.exe",
    "ar": "C:/JL/mc/bin/mc-uclinux-ar.exe",
    "cc_opts": [],
    "ld_opts": [],
    "inc_dirs": [
      "C:/JL/mc/include"
    ],
    "lib_dirs": [
      "C:/JL/mc/lib"
    ],
    "libs": [
      "C:/JL/mc/lib/libc.a",
      "C:/JL/mc/lib/libgcc.a",
      "C:/JL/mc/lib/libgcov.a",
      "C:/JL/mc/lib/libiberty.a",
      "C:/JL/mc/lib/libm.a"
    ]
  },
  "pi32v2_r2_large_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r2",
      "-integrated-as",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r2"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r2-large"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r2-large/libc.a",
      "C:/JL/pi32/pi32v2-lib/r2-large/libm.a",
      "C:/JL/pi32/pi32v2-lib/r2-large/libcompiler-rt.a"
    ]
  }
}


def convert_path(xs):
    return list(map(lambda x: x.replace('\\', '/'), xs))

def append_line(lines, v):
    lines.append('\t{} \\'.format(v))

def convert_path_except_first(xs):
    if len(xs) <= 1:
        return xs
    return xs[:1] + convert_path(xs[1:])

def convert_cmd_path(xs):
    return list(map(lambda x: ' '.join(convert_path_except_first(x.split())), xs))

def output_var_def(lines, var, vals, conv=None):
    if len(vals) == 0:
        lines.append('{} :='.format(var))
        lines.append('')
        lines.append('')
    else:
        lines.append('{} := \\'.format(var))
        for c in vals:
            if conv:
                append_line(lines, conv(c))
            else:
                append_line(lines, c)
        lines.append('')
        lines.append('')

def conv_vars(line):
    line = line.replace('$compiler', '$(CC)')
    line = line.replace('$options', '$(CFLAGS) $(DEFINES)')
    line = line.replace('$includes', '$(INCLUDES)')
    return line

def get_rela(path):
    i = 0
    rela = ''
    while i < len(path):
        if path[i:i+3] == '../':
            rela += path[i:i+3]
            i += 3
        else:
            break
    if rela != '' and rela[-1] == '/':
        rela = rela[:-1]
    return rela

def get_root_prefix(paths):
    max_rela = ''
    for p in paths:
        rela = get_rela(p)
        if len(rela) > len(max_rela):
            max_rela = rela
    return max_rela

def remove_tailing_slash(path):
    if path.endswith('/'):
        return path[:-1]
    return path

def unify_root_prefix(root_prefix, path):
    if path.startswith(root_prefix):
        return path
    rela = get_rela(path)
    count = (root_prefix.count('.') - rela.count('.')) // 2
    if count == 0:
        return path

    if rela != '':
        upath = path[len(rela)+1:]
    else:
        upath = path
    first_comp = upath.split('/')[0]
    if rela != '':
        upath = rela + '/' + first_comp
    else:
        upath = first_comp

    abspath = os.path.abspath(upath)
    ds = []
    for i in range(0, count):
        abspath = os.path.dirname(abspath)
        d = os.path.basename(abspath)
        ds.append(d)
    ds.reverse()
    if rela != '':
        return root_prefix + '/' + '/'.join(ds) + path[len(rela):]
    else:
        return root_prefix + '/' + '/'.join(ds) + '/' + path

def unify_root_prefix_of_list(root_prefix, paths):
    ret = []
    for path in paths:
        ret.append(unify_root_prefix(root_prefix, path))
    return ret

BUILD_TYPE_EXE = "exe"
BUILD_TYPE_STATIC_LIB = "static_lib"

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='CBP2MAKE')
    parser.add_argument('--cbp', type=str, default='', help='path to .cbp file')
    parser.add_argument('--output', type=str, default='', help='output Makefile path')
    parser.add_argument('--builddir', type=str, default='objs', help='BUILD_DIR')

    args = parser.parse_args(sys.argv[1:])

    args.cbp = os.path.abspath(args.cbp)
    args.output = os.path.abspath(args.output)

    os.chdir(os.path.dirname(args.cbp))

    with open(args.cbp, 'r') as fin:
        cbp = ET.fromstring(fin.read())

    options = cbp.findall('Project/Option')
    units = cbp.findall('Project/Unit')
    build = cbp.find('Project/Build/Target')

    compiler = None
    title = None

    for o in options:
        title = o.get('title', title)
        compiler = o.get('compiler', compiler)

    build_opts = build.findall('Option')
    for b in build_opts:
        comp = b.get('compiler')
        if comp != None:
            compiler = comp
            break

    if (title is None) or (compiler is None):
        print('title or compiler not found')
        sys.exit(1)

    if compiler not in COMPILERS:
        print('compiler {} not supported'.format(compiler))
        sys.exit(1)

    build_opts = build.findall('Option')

    build_output = None
    build_prefix_auto = None
    build_extension_auto = None
    build_type = None
    for b in build_opts:
        build_output = b.get('output', build_output)
        build_prefix_auto = b.get('prefix_auto', build_prefix_auto)
        build_extension_auto = b.get('extension_auto', build_extension_auto)
        build_type = b.get('type', build_type)
  
    output_suffix = ''
    if build_type is None or build_type == "1":
      # building executable
      build_type = BUILD_TYPE_EXE
      output_suffix = '.elf'
    elif build_type == "2":
      build_type = BUILD_TYPE_STATIC_LIB
      output_suffix = '.a'
    else:
      print('unknown build_type {}'.format(build_type))
      sys.exit(1)

    if build_output is None:
        print('output not found')
        sys.exit(1)

    if build_prefix_auto == "1" and build_extension_auto == "1":
      build_output = build_output + '/' + title + output_suffix

    cbp_cc_opts = []
    cbp_def_opts = []
    cbp_inc_dirs = []
    for c in build.findall('Compiler/Add'):
        opt = c.get('option')
        inc = c.get('directory')
        if opt:
            if opt.startswith('-D'):
                opt = opt.replace('"', '\\"')
                cbp_def_opts.append(opt)
            else:
                cbp_cc_opts.append(opt)
        if inc:
            cbp_inc_dirs.append(inc)
    cbp_cc_opts = convert_path(cbp_cc_opts)
    cbp_inc_dirs = convert_path(cbp_inc_dirs)

    cbp_link_opts = []
    cbp_link_libs = []
    for c in build.findall('Linker/Add'):
        opt = c.get('option')
        lib = c.get('library')
        if opt:
            cbp_link_opts.append(opt)
        if lib:
            cbp_link_libs.append(lib)
    cbp_link_opts = convert_path(cbp_link_opts)
    cbp_link_libs = convert_path(cbp_link_libs)

    cbp_pre_builds = []
    cbp_post_builds = []
    for c in build.findall('ExtraCommands/Add'):
        before = c.get('before')
        after = c.get('after')
        if before:
            cbp_pre_builds.append(conv_vars(before))
        if after:
            cbp_post_builds.append(conv_vars(after))
    cbp_pre_builds = convert_cmd_path(cbp_pre_builds)
    cbp_post_builds = convert_cmd_path(cbp_post_builds)
    fix_bat_cmd = None
    for c in cbp_pre_builds:
      cs = c.split()
      if 'fixbat.exe' in cs[0]:
        fix_bat_cmd = cs[0]

    post_script = ''
    post_script_linux = ''
    if len(cbp_post_builds) != 0:
      post_script = cbp_post_builds[0].split()[0]
      post_script_conv = post_script.replace('\\', '/')
      post_script_linux = '.'.join(post_script_conv.split('.')[:-1] + ['sh'])

      cbp_pre_builds = list(map(lambda x: x.replace(post_script_conv, "$(POST_SCRIPT)"),
                                cbp_pre_builds))
      cbp_post_builds = list(map(lambda x: x.replace(post_script, "$(RUN_POST_SCRIPT)"),
                                cbp_post_builds))

    # if cbp_pre_builds contains 'fixbat.exe', we should replace it by $(FIXBAT)
    if fix_bat_cmd is not None:
      cbp_pre_builds = list(map(lambda x: x.replace(fix_bat_cmd, '$(FIXBAT)'),
                            cbp_pre_builds))

    cbp_c_files = []
    cbp_s_files = []
    cbp_S_files = []
    cbp_cpp_files = []
    cbp_cc_files = []
    cbp_cxx_files = []
    cxx_extra_flags = []

    for u in units:
        filename = u.get('filename')
        is_cxx_file = False
        if filename:
            filename = filename.replace('\\', '/')
            if filename.endswith('.c'):
                cbp_c_files.append(filename)
            elif filename.endswith('.S'):
                cbp_S_files.append(filename)
            elif filename.endswith('.s'):
                cbp_s_files.append(filename)
            elif filename.endswith('.cpp'):
                is_cxx_file = True
                cbp_cpp_files.append(filename)
            elif filename.endswith('.cxx'):
                is_cxx_file = True
                cbp_cxx_files.append(filename)
            elif filename.endswith('.cc'):
                is_cxx_file = True
                cbp_cc_files.append(filename)
        
        if is_cxx_file:
            # 对于 C++, C 混合编译的情况，需要单独提取出 C++ 的编译参数，例如 -std=c++*
            for fileopt in u.findall('Option'):
                buildCommand = fileopt.get('buildCommand')
                if buildCommand:
                    # should extract cxxflags:
                    bcs = buildCommand.split()
                    for b in bcs:
                        if b in ['$compiler', '$options', '$includes', '-c', '$file', '-o', '$object']:
                            continue
                        if b not in cxx_extra_flags:
                            cxx_extra_flags.append(b)

    comp = COMPILERS[compiler]

    comp_cc_opts = comp['cc_opts']
    comp_inc_dirs = list(map(remove_tailing_slash, comp['inc_dirs']))
    comp_ld_opts = comp['ld_opts']
    comp_lib_dirs = list(map(remove_tailing_slash, comp['lib_dirs']))
    comp_libs = comp['libs']

    root_prefix = get_root_prefix(cbp_c_files + cbp_S_files + cbp_s_files + cbp_cpp_files)
    if root_prefix != '':
        cbp_c_files = unify_root_prefix_of_list(root_prefix, cbp_c_files)
        cbp_s_files = unify_root_prefix_of_list(root_prefix, cbp_s_files)
        cbp_S_files = unify_root_prefix_of_list(root_prefix, cbp_S_files)
        cbp_cpp_files = unify_root_prefix_of_list(root_prefix, cbp_cpp_files)

    # 有些时候，可能这些文件，并不在同一个目录下，需要归一化，例如
    # ../../common/a.c
    # ../b.c
    # a.c
    # 需要统一到最长的前缀，例如
    # ../../common/a.c
    # ../../code/b.c
    # ../../code/main/a.c
    

    lines = []

    lines.append("""
# make 编译并下载
# make VERBOSE=1 显示编译详细过程
# make clean 清除编译临时文件
#
# 注意： Linux 下编译方式：
#     1. 从 http://pkgman.jieliapp.com/doc/all 处找到下载链接
#     2. 下载后，解压到 /opt/jieli 目录下，保证
#       /opt/jieli/common/bin/clang 存在（注意目录层次）
#     3. 确认 ulimit -n 的结果足够大（建议大于8096），否则链接可能会因为打开文件太多而失败
#       可以通过 ulimit -n 8096 来设置一个较大的值
#
""")

    arch = compiler.split('_')[0]
    rev = ''
    if arch == 'pi32v2':
        cs = compiler.split('_')
        if len(cs) > 1:
            if cs[1] in ['r1','r2','r3']:
                rev = cs[1]
                if cs[2] == 'large':
                  rev += '-large'

    # TOOLS PATH SETTINGS
    lines.append('# 工具路径设置')

    lines.append('ifeq ($(OS), Windows_NT)')
    lines.append('# Windows 下工具链位置')
    lines.append('TOOL_DIR := {}'.format('/'.join(comp['cc'].split('/')[:-1])))
    lines.append('CC    := {}'.format(comp['cc'].split('/')[-1]))
    lines.append('CXX   := {}'.format(comp['cxx'].split('/')[-1]))
    lines.append('LD    := {}'.format(comp['ld'].split('/')[-1]))
    lines.append('AR    := {}'.format(comp['ar'].split('/')[-1]))
    lines.append('MKDIR := mkdir_win -p')
    lines.append('RM    := rm -rf')
    lines.append('')
    lines.append('SYS_LIB_DIR := {}'.format(comp_lib_dirs[0]))
    lines.append('SYS_INC_DIR := {}'.format(comp_inc_dirs[0]))
    lines.append('EXT_CFLAGS  := # Windows 下不需要 -D__SHELL__')
    lines.append('export PATH:=$(TOOL_DIR);$(PATH)')
    lines.append('')
    lines.append('## 后处理脚本')
    if fix_bat_cmd is not None:
      lines.append('FIXBAT          := {} # 用于处理 utf8->gbk 编码问题'.format(fix_bat_cmd))
    lines.append('POST_SCRIPT     := {}'.format(post_script))
    lines.append('RUN_POST_SCRIPT := $(POST_SCRIPT)')

    lines.append('else')
    lines.append('# Linux 下工具链位置')
    lines.append('TOOL_DIR := /opt/jieli/{}/bin'.format(arch))
    lines.append('CC    := clang')
    lines.append('CXX   := clang++')
    lines.append('LD    := lto-wrapper')
    lines.append('AR    := lto-ar')
    lines.append('MKDIR := mkdir -p')
    lines.append('RM    := rm -rf')
    # Linux 下，增加 OBJDUMP, OBJCOPY, OBJSIZEDUMP 的定义
    lines.append('export OBJDUMP := $(TOOL_DIR)/objdump')
    lines.append('export OBJCOPY := $(TOOL_DIR)/objcopy')
    lines.append('export OBJSIZEDUMP := $(TOOL_DIR)/objsizedump')
    lines.append('')
    if rev == '':
        lines.append('SYS_LIB_DIR := $(TOOL_DIR)/../lib')
    else:
        lines.append('SYS_LIB_DIR := $(TOOL_DIR)/../lib/{}'.format(rev))
    lines.append('SYS_INC_DIR := $(TOOL_DIR)/../include')
    lines.append('EXT_CFLAGS  := -D__SHELL__ # Linux 下需要这个保证正确处理 download.c')
    lines.append('export PATH:=$(TOOL_DIR):$(PATH)')
    lines.append('')

    lines.append('## 后处理脚本')
    if fix_bat_cmd is not None:
      lines.append('FIXBAT          := touch # Linux下不需要处理 bat 编码问题')
    lines.append('POST_SCRIPT     := {}'.format(post_script_linux))
    lines.append('RUN_POST_SCRIPT := bash $(POST_SCRIPT)')

    lines.append('endif')
    lines.append('')

    lines.append('CC  := $(TOOL_DIR)/$(CC)')
    lines.append('CXX := $(TOOL_DIR)/$(CXX)')
    lines.append('LD  := $(TOOL_DIR)/$(LD)')
    lines.append('AR  := $(TOOL_DIR)/$(AR)')


    lines.append('# 输出文件设置')
    lines.append('OUT_ELF   := {}'.format(build_output))
    lines.append('OBJ_FILE  := $(OUT_ELF).objs.txt')
    lines.append('# 编译路径设置')
    lines.append('BUILD_DIR := {}'.format(args.builddir))

    if root_prefix != '':
        lines.append('# 工程路径前缀')
        lines.append('ROOT_PREFIX := {}'.format(root_prefix))
    lines.append('')

    lines.append('# 编译参数设置')
    output_var_def(lines, 'CFLAGS', comp_cc_opts + cbp_cc_opts)
    lines.append('# C++额外的编译参数')
    output_var_def(lines, 'CXXFLAGS', cxx_extra_flags)
    lines.append('# 宏定义')
    output_var_def(lines, 'DEFINES', cbp_def_opts)
    lines.append('DEFINES += $(EXT_CFLAGS) # 额外的一些定义\n')
    lines.append('# 头文件搜索路径')
    output_var_def(lines, 'INCLUDES', cbp_inc_dirs + ['$(SYS_INC_DIR)'], lambda x : '-I' + x)

    lines.append('# 需要编译的 .c 文件')
    output_var_def(lines, 'c_SRC_FILES', cbp_c_files)

    lines.append('# 需要编译的 .S 文件')
    output_var_def(lines, 'S_SRC_FILES', cbp_S_files)

    lines.append('# 需要编译的 .s 文件')
    output_var_def(lines, 's_SRC_FILES', cbp_s_files)

    lines.append('# 需要编译的 .cpp 文件')
    output_var_def(lines, 'cpp_SRC_FILES', cbp_cpp_files)

    lines.append('# 需要编译的 .cc 文件')
    output_var_def(lines, 'cc_SRC_FILES', cbp_cc_files)

    lines.append('# 需要编译的 .cxx 文件')
    output_var_def(lines, 'cxx_SRC_FILES', cbp_cxx_files)

    lines.append('# 链接参数')
    if build_type == BUILD_TYPE_EXE:
      output_var_def(lines, 'LFLAGS', cbp_link_opts + comp_ld_opts)
      output_var_def(lines, 'LIBPATHS', ['$(SYS_LIB_DIR)'], lambda x : '-L' + x)
      output_var_def(lines, 'LIBS',
          list(map(lambda x: x.replace(comp_lib_dirs[0], '$(SYS_LIB_DIR)'), cbp_link_libs + comp_libs)))

    lines.append("""
c_OBJS    := $(c_SRC_FILES:%.c=%.c.o)
S_OBJS    := $(S_SRC_FILES:%.S=%.S.o)
s_OBJS    := $(s_SRC_FILES:%.s=%.s.o)
cpp_OBJS  := $(cpp_SRC_FILES:%.cpp=%.cpp.o)
cxx_OBJS  := $(cxx_SRC_FILES:%.cxx=%.cxx.o)
cc_OBJS   := $(cc_SRC_FILES:%.cc=%.cc.o)

OBJS      := $(c_OBJS) $(S_OBJS) $(s_OBJS) $(cpp_OBJS) $(cxx_OBJS) $(cc_OBJS)
DEP_FILES := $(OBJS:%.o=%.d)
""")

    if root_prefix != '':
        lines.append("""
OBJS      := $(addprefix $(BUILD_DIR)/, $(OBJS:$(ROOT_PREFIX)/%=%))
DEP_FILES := $(addprefix $(BUILD_DIR)/, $(DEP_FILES:$(ROOT_PREFIX)/%=%))
""")
    else:
        lines.append("""
OBJS      := $(addprefix $(BUILD_DIR)/, $(OBJS))
DEP_FILES := $(addprefix $(BUILD_DIR)/, $(DEP_FILES))
""")

    lines.append("""
VERBOSE ?= 0
ifeq ($(VERBOSE), 1)
QUITE :=
else
QUITE := @
endif

# 一些旧的 make 不支持 file 函数，需要 make 的时候指定 LINK_AT=0 make
LINK_AT ?= 1

# 表示下面的不是一个文件的名字，无论是否存在 all, clean, pre_build 这样的文件
# 还是要执行命令
# see: https://www.gnu.org/software/make/manual/html_node/Phony-Targets.html
.PHONY: all clean pre_build

# 不要使用 make 预设置的规则
# see: https://www.gnu.org/software/make/manual/html_node/Suffix-Rules.html
.SUFFIXES:

all: pre_build $(OUT_ELF)
	$(info +POST-BUILD)""")
    # 这里不用 echo，为了兼容起见。一些 Windows 环境下， echo 有问题

    # and append post build
    for p in cbp_post_builds:
        lines.append('\t$(QUITE) {}'.format(p))

    lines.append('')
    lines.append('pre_build:')
    lines.append('\t$(info +PRE-BUILD)')
    for p in cbp_pre_builds:
        lines.append('\t$(QUITE) {}'.format(p))

    if root_prefix != '':
        rp = '$(ROOT_PREFIX)/'
    else:
        rp = ''

    lines.append("""
clean:
	$(QUITE) $(RM) $(OUT_ELF)
	$(QUITE) $(RM) $(BUILD_DIR)

""")
    if build_type == BUILD_TYPE_EXE:
      lines.append("""
ifeq ($(LINK_AT), 1)
$(OUT_ELF): $(OBJS)
	$(info +LINK $@)
	$(shell $(MKDIR) $(@D))
	$(file >$(OBJ_FILE), $(OBJS))
	$(QUITE) $(LD) -o $(OUT_ELF) @$(OBJ_FILE) $(LFLAGS) $(LIBPATHS) $(LIBS)
else
$(OUT_ELF): $(OBJS)
	$(info +LINK $@)
	$(shell $(MKDIR) $(@D))
	$(QUITE) $(LD) -o $(OUT_ELF) $(OBJS) $(LFLAGS) $(LIBPATHS) $(LIBS)
endif
""")
    elif build_type == BUILD_TYPE_STATIC_LIB:
      lines.append("""
ifeq ($(LINK_AT), 1)
$(OUT_ELF): $(OBJS)
	$(info +AR $@)
	$(shell $(MKDIR) $(@D))
	$(file >$(OBJ_FILE), $(OBJS))
	$(QUITE) $(AR) rs $(OUT_ELF) @$(OBJ_FILE)
else
$(OUT_ELF): $(OBJS)
	$(info +AR $@)
	$(shell $(MKDIR) $(@D))
	$(QUITE) $(AR) rs $(OUT_ELF) $(OBJS)
endif
""")

    lines.append("""
$(BUILD_DIR)/%.c.o : {rp}%.c
	$(info +CC $<)
	@$(MKDIR) $(@D)
	@$(CC) $(CFLAGS) $(DEFINES) $(INCLUDES) -MM -MT $@ $< -o $(@:.o=.d)
	$(QUITE) $(CC) $(CFLAGS) $(DEFINES) $(INCLUDES) -c $< -o $@

$(BUILD_DIR)/%.S.o : {rp}%.S
	$(info +AS $<)
	@$(MKDIR) $(@D)
	@$(CC) $(CFLAGS) $(DEFINES) $(INCLUDES) -MM -MT $@ $< -o $(@:.o=.d)
	$(QUITE) $(CC) $(CFLAGS) $(DEFINES) $(INCLUDES) -c $< -o $@

$(BUILD_DIR)/%.s.o : {rp}%.s
	$(info +AS $<)
	@$(MKDIR) $(@D)
	@$(CC) $(CFLAGS) $(DEFINES) $(INCLUDES) -MM -MT $@ $< -o $(@:.o=.d)
	$(QUITE) $(CC) $(CFLAGS) $(DEFINES) $(INCLUDES) -c $< -o $@

$(BUILD_DIR)/%.cpp.o : {rp}%.cpp
	$(info +CXX $<)
	@$(MKDIR) $(@D)
	@$(CXX) $(CFLAGS) $(CXXFLAGS) $(DEFINES) $(INCLUDES) -MM -MT $@ $< -o $(@:.o=.d)
	$(QUITE) $(CXX) $(CXXFLAGS) $(CFLAGS) $(DEFINES) $(INCLUDES) -c $< -o $@

$(BUILD_DIR)/%.cxx.o : {rp}%.cxx
	$(info +CXX $<)
	@$(MKDIR) $(@D)
	@$(CXX) $(CFLAGS) $(CXXFLAGS) $(DEFINES) $(INCLUDES) -MM -MT $@ $< -o $(@:.o=.d)
	$(QUITE) $(CXX) $(CXXFLAGS) $(CFLAGS) $(DEFINES) $(INCLUDES) -c $< -o $@

$(BUILD_DIR)/%.cc.o : {rp}%.cc
	$(info +CXX $<)
	@$(MKDIR) $(@D)
	@$(CXX) $(CFLAGS) $(CXXFLAGS) $(DEFINES) $(INCLUDES) -MM -MT $@ $< -o $(@:.o=.d)
	$(QUITE) $(CXX) $(CXXFLAGS) $(CFLAGS) $(DEFINES) $(INCLUDES) -c $< -o $@

-include $(DEP_FILES)
""".format(rp=rp))

    with open(args.output, 'w', encoding='utf-8') as fout:
        fout.write('\n'.join(lines))