#!/usr/bin/env python3

# 把 cbp 转换为一个 vscode 的 .vscode/c_cpp_properties.json 文件

import argparse
import sys
import os
import xml.etree.ElementTree as ET
import json

COMPILERS = {
  "dv10": {
    "cc": "C:/JL/dv10/bin/dv10-elf-gcc.exe",
    "cxx": "C:/JL/dv10/bin/dv10-elf-g++.exe",
    "ld": "C:/JL/dv10/bin/dv10-elf-ld.exe",
    "ar": "C:/JL/dv10/bin/dv10-elf-ar.exe",
    "cc_opts": [],
    "ld_opts": [],
    "inc_dirs": [
      "C:/JL/dv10/include"
    ],
    "lib_dirs": [
      "C:/JL/dv10/lib"
    ],
    "libs": [
      "C:/JL/dv10/lib/libc.a",
      "C:/JL/dv10/lib/libgcc.a",
      "C:/JL/dv10/lib/libgcov.a",
      "C:/JL/dv10/lib/libiberty.a",
      "C:/JL/dv10/lib/libm.a"
    ],
  },
  "dv12": {
    "cc": "C:/JL/dv12/bin/dv12-elf-gcc.exe",
    "cxx": "C:/JL/dv12/bin/dv12-elf-g++.exe",
    "ld": "C:/JL/dv12/bin/dv12-elf-ld.exe",
    "ar": "C:/JL/dv12/bin/dv12-elf-ar.exe",
    "cc_opts": [],
    "ld_opts": [],
    "inc_dirs": [
      "C:/JL/dv12/include"
    ],
    "lib_dirs": [
      "C:/JL/dv12/lib"
    ],
    "libs": [
      "C:/JL/dv12/lib/libc.a",
      "C:/JL/dv12/lib/libgcc.a",
      "C:/JL/dv12/lib/libgcov.a",
      "C:/JL/dv12/lib/libiberty.a",
      "C:/JL/dv12/lib/libm.a"
    ]
  },
  "pi32_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/lto-ar.exe",
    "cc_opts": [
      "-target pi32",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -pi32-memreg-opt",
      "-mllvm -pi32-mem-offset-adj-opt",
      "-mllvm -pi32-const-spill",
      "-mllvm -pi32-enable-jz",
      "-mllvm -pi32-tailcall-opt",
      "-mllvm -inline-threshold=5",
      "-mllvm -pi32-enable-itblock=1",
      "-Oz"
    ],
    "ld_opts": [
      "--plugin-opt=-pi32-memreg-opt",
      "--plugin-opt=-pi32-mem-offset-adj-opt",
      "--plugin-opt=-pi32-const-spill",
      "--plugin-opt=-pi32-enable-jz",
      "--plugin-opt=-pi32-tailcall-opt",
      "--plugin-opt=-inline-threshold=5",
      "--plugin-opt=-pi32-enable-itblock=1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/include/libc"
    ],
    "lib_dirs": [
      "C:/JL/pi32/libc"
    ],
    "libs": [
      "C:/JL/pi32/lib/libc.a",
      "C:/JL/pi32/lib/libm.a",
      "C:/JL/pi32/lib/libcompiler-rt.a"
    ],
    "defs": ["__PI32__"],
  },
  "pi32_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/lto-ar.exe",
    "cc_opts": [
      "-target pi32",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -pi32-memreg-opt",
      "-mllvm -pi32-mem-offset-adj-opt",
      "-mllvm -pi32-const-spill",
      "-mllvm -pi32-enable-jz",
      "-mllvm -pi32-tailcall-opt",
      "-mllvm -inline-threshold=5",
      "-mllvm -pi32-enable-itblock=1",
      "-Oz",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=-pi32-memreg-opt",
      "--plugin-opt=-pi32-mem-offset-adj-opt",
      "--plugin-opt=-pi32-const-spill",
      "--plugin-opt=-pi32-enable-jz",
      "--plugin-opt=-pi32-tailcall-opt",
      "--plugin-opt=-inline-threshold=5",
      "--plugin-opt=-pi32-enable-itblock=1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/include/libc"
    ],
    "lib_dirs": [
      "C:/JL/pi32/libc"
    ],
    "libs": [
      "C:/JL/pi32/lib/libc.a",
      "C:/JL/pi32/lib/libm.a",
      "C:/JL/pi32/lib/libcompiler-rt.a"
    ],
    "defs": ["__PI32__"],
  },
  "pi32_lto_compiler_without_libc": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/lto-ar.exe",
    "cc_opts": [
      "-target pi32",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -pi32-memreg-opt",
      "-mllvm -pi32-mem-offset-adj-opt",
      "-mllvm -pi32-const-spill",
      "-mllvm -pi32-enable-jz",
      "-mllvm -pi32-tailcall-opt",
      "-mllvm -inline-threshold=5",
      "-mllvm -pi32-enable-itblock=1",
      "-Oz",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=-pi32-memreg-opt",
      "--plugin-opt=-pi32-mem-offset-adj-opt",
      "--plugin-opt=-pi32-const-spill",
      "--plugin-opt=-pi32-enable-jz",
      "--plugin-opt=-pi32-tailcall-opt",
      "--plugin-opt=-inline-threshold=5",
      "--plugin-opt=-pi32-enable-itblock=1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/include/libc"
    ],
    "lib_dirs": [
      "C:/JL/pi32/libc"
    ],
    "libs": [
      "C:/JL/pi32/lib/libcompiler-rt.a"
    ],
    "defs": ["__PI32__"],
  },
  "pi32_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/lto-ar.exe",
    "cc_opts": [
      "-target pi32",
      "-integrated-as"
    ],
    "ld_opts": [],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": [],
    "defs": ["__PI32__"],
  },
  "pi32v2_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -inline-threshold=5",
      "-Oz"
    ],
    "ld_opts": [
      "--plugin-opt=-inline-threshold=5"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/libc.a",
      "C:/JL/pi32/pi32v2-lib/libm.a",
      "C:/JL/pi32/pi32v2-lib/libcompiler-rt.a"
    ],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -inline-threshold=5",
      "-Oz",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=-inline-threshold=5"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/libc.a",
      "C:/JL/pi32/pi32v2-lib/libm.a",
      "C:/JL/pi32/pi32v2-lib/libcompiler-rt.a"
    ],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_lto_compiler_without_libc": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-flto",
      "-target pi32v2",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -inline-threshold=5",
      "-Oz"
    ],
    "ld_opts": [
      "-flto",
      "--plugin-opt=-inline-threshold=5"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/libcompiler-rt.a"
    ],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as"
    ],
    "ld_opts": [],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": [],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_r1_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as",
      "-mcpu=r1"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r1"
    ],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": [],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_r1_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r1",
      "-integrated-as",
      "-mllvm -inline-threshold=5",
      "-Oz",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=-inline-threshold=5",
      "--plugin-opt=mcpu=r1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r1"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r1/libc.a",
      "C:/JL/pi32/pi32v2-lib/r1/libm.a",
      "C:/JL/pi32/pi32v2-lib/r1/libcompiler-rt.a"
    ],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_r1_large_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r1",
      "-integrated-as",
      "-mllvm -inline-threshold=5",
      "-Oz",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=-inline-threshold=5",
      "--plugin-opt=mcpu=r1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r1-large"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r1-large/libc.a",
      "C:/JL/pi32/pi32v2-lib/r1-large/libm.a",
      "C:/JL/pi32/pi32v2-lib/r1-large/libcompiler-rt.a"
    ],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_r2_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as",
      "-mcpu=r2"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r2"
    ],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": [],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_r2_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r2",
      "-integrated-as",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r2"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r2"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r2/libc.a",
      "C:/JL/pi32/pi32v2-lib/r2/libm.a",
      "C:/JL/pi32/pi32v2-lib/r2/libcompiler-rt.a"
    ],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_r3_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as",
      "-mcpu=r3",
      "-mfprev1"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r3",
      "--plugin-opt=-mattr=+fprev1"
    ],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": [],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_r3_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r3",
      "-integrated-as",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r3",
      "--plugin-opt=-mattr=+fprev1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r3"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r3/libc.a",
      "C:/JL/pi32/pi32v2-lib/r3/libm.a",
      "C:/JL/pi32/pi32v2-lib/r3/libcompiler-rt.a"
    ],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_r3_large_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r3",
      "-integrated-as",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r3",
      "--plugin-opt=-mattr=+fprev1"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r3-large"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r3-large/libc.a",
      "C:/JL/pi32/pi32v2-lib/r3-large/libm.a",
      "C:/JL/pi32/pi32v2-lib/r3-large/libcompiler-rt.a"
    ],
    "defs": ["__PI32V2__"],
  },
  "q32s_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/q32s-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/q32s-lto-ar.exe",
    "cc_opts": [
      "-target q32s",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -inline-threshold=5",
      "-Oz"
    ],
    "ld_opts": [
      "--plugin-opt=-inline-threshold=5"
    ],
    "inc_dirs": [
      "C:/JL/pi32/q32s-include"
    ],
    "lib_dirs": [
      "C:/JL/pi32/q32s-lib"
    ],
    "libs": [
      "C:/JL/pi32/q32s-lib/libc.a",
      "C:/JL/pi32/q32s-lib/libm.a",
      "C:/JL/pi32/q32s-lib/libcompiler-rt.a"
    ],
    "defs": ["__Q32S__"],
  },
  "pi32v2_r5_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-integrated-as",
      "-mcpu=r5",
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r5",
    ],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": [],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_r5_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r5",
      "-integrated-as",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r5",
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r5"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r5/libc.a",
      "C:/JL/pi32/pi32v2-lib/r5/libm.a",
      "C:/JL/pi32/pi32v2-lib/r5/libcompiler-rt.a"
    ],
    "defs": ["__PI32V2__"],
  },
  "pi32v2_r5_large_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r5",
      "-integrated-as",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r5",
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r5-large"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r5-large/libc.a",
      "C:/JL/pi32/pi32v2-lib/r5-large/libm.a",
      "C:/JL/pi32/pi32v2-lib/r5-large/libcompiler-rt.a"
    ],
    "defs": ["__PI32V2__"],
  },
  "q32s_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/q32s-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/q32s-lto-ar.exe",
    "cc_opts": [
      "-flto",
      "-target q32s",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -inline-threshold=5",
      "-Oz"
    ],
    "ld_opts": [
      "-flto",
      "--plugin-opt=-inline-threshold=5"
    ],
    "inc_dirs": [
      "C:/JL/pi32/q32s-include"
    ],
    "lib_dirs": [
      "C:/JL/pi32/q32s-lib"
    ],
    "libs": [
      "C:/JL/pi32/q32s-lib/libc.a",
      "C:/JL/pi32/q32s-lib/libm.a",
      "C:/JL/pi32/q32s-lib/libcompiler-rt.a"
    ],
    "defs": ["__Q32S__"],
  },
  "q32s_lto_compiler_without_libc": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/q32s-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/q32s-lto-ar.exe",
    "cc_opts": [
      "-flto",
      "-target q32s",
      "-integrated-as",
      "-fno-builtin",
      "-mllvm -inline-threshold=5",
      "-Oz"
    ],
    "ld_opts": [
      "-flto",
      "--plugin-opt=-inline-threshold=5"
    ],
    "inc_dirs": [
      "C:/JL/pi32/q32s-include"
    ],
    "lib_dirs": [
      "C:/JL/pi32/q32s-lib"
    ],
    "libs": [
      "C:/JL/pi32/q32s-lib/libcompiler-rt.a"
    ],
    "defs": ["__Q32S__"],
  },
  "q32s_empty_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/q32s-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/q32s-lto-ar.exe",
    "cc_opts": [
      "-target q32s",
      "-integrated-as"
    ],
    "ld_opts": [],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": [],
    "defs": ["__Q32S__"],
  },
  "f95_assembler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/f95-ld.exe",
    "ar": "C:/JL/pi32/bin/f95-ar.exe",
    "cc_opts": [
      "-target f95",
      "-integrated-as"
    ],
    "ld_opts": [],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": []
  },
  "f96_assembler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/f95-ld.exe",
    "ar": "C:/JL/pi32/bin/f95-ar.exe",
    "cc_opts": [
      "-target f95",
      "-integrated-as",
      "-mcpu=r1"
    ],
    "ld_opts": [],
    "inc_dirs": [],
    "lib_dirs": [],
    "libs": []
  },
  "ac46_compiler": {
    "cc": "C:/JL/ac46/bin/ac46-elf-gcc.exe",
    "cxx": "C:/JL/ac46/bin/ac46-elf-g++.exe",
    "ld": "C:/JL/ac46/bin/ac46-elf-ld.exe",
    "ar": "C:/JL/ac46/bin/ac46-elf-ar.exe",
    "cc_opts": [],
    "ld_opts": [],
    "inc_dirs": [
      "C:/JL/ac46/ac46-elf/include"
    ],
    "lib_dirs": [
      "C:/JL/ac46/ac46-elf/lib"
    ],
    "libs": [
      "C:/JL/ac46/ac46-elf/lib/libc.a",
      "C:/JL/ac46/ac46-elf/lib/libgcc.a",
      "C:/JL/ac46/ac46-elf/lib/libgcov.a",
      "C:/JL/ac46/ac46-elf/lib/libm.a"
    ]
  },
  "mc": {
    "cc": "C:/JL/mc/bin/mc-uclinux-gcc.exe",
    "cxx": "C:/JL/mc/bin/mc-uclinux-g++.exe",
    "ld": "C:/JL/mc/bin/mc-uclinux-ld.exe",
    "ar": "C:/JL/mc/bin/mc-uclinux-ar.exe",
    "cc_opts": [],
    "ld_opts": [],
    "inc_dirs": [
      "C:/JL/mc/include"
    ],
    "lib_dirs": [
      "C:/JL/mc/lib"
    ],
    "libs": [
      "C:/JL/mc/lib/libc.a",
      "C:/JL/mc/lib/libgcc.a",
      "C:/JL/mc/lib/libgcov.a",
      "C:/JL/mc/lib/libiberty.a",
      "C:/JL/mc/lib/libm.a"
    ]
  },
  "pi32v2_r2_large_lto_compiler": {
    "cc": "C:/JL/pi32/bin/clang.exe",
    "cxx": "C:/JL/pi32/bin/clang.exe",
    "ld": "C:/JL/pi32/bin/pi32v2-lto-wrapper.exe",
    "ar": "C:/JL/pi32/bin/pi32v2-lto-ar.exe",
    "cc_opts": [
      "-target pi32v2",
      "-mcpu=r2",
      "-integrated-as",
      "-flto"
    ],
    "ld_opts": [
      "--plugin-opt=mcpu=r2"
    ],
    "inc_dirs": [
      "C:/JL/pi32/pi32v2-include/"
    ],
    "lib_dirs": [
      "C:/JL/pi32/pi32v2-lib/r2-large"
    ],
    "libs": [
      "C:/JL/pi32/pi32v2-lib/r2-large/libc.a",
      "C:/JL/pi32/pi32v2-lib/r2-large/libm.a",
      "C:/JL/pi32/pi32v2-lib/r2-large/libcompiler-rt.a"
    ],
    "defs": ["__PI32V2__"],
  }
}


def convert_path(xs):
    return list(map(lambda x: x.replace('\\', '/'), xs))

def map_to_root_path(rootdir, cbppath, path):
    path = os.path.join(os.path.dirname(cbppath), path)
    path = os.path.realpath(path)
    path = os.path.relpath(path, start=rootdir)
    return path

def remove_tailing_slash(path):
    if path.endswith('/'):
        return path[:-1]
    return path


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='CBP2MAKE')
    parser.add_argument('--cbp', type=str, default='', help='path to .cbp file')
    parser.add_argument('--root', type=str, default='.', help='path to root directory')

    args = parser.parse_args(sys.argv[1:])

    with open(args.cbp, 'r') as fin:
        cbp = ET.fromstring(fin.read())

    options = cbp.findall('Project/Option')
    units = cbp.findall('Project/Unit')
    build = cbp.find('Project/Build/Target')

    compiler = None
    title = None

    for o in options:
        title = o.get('title', title)
        compiler = o.get('compiler', compiler)

    if (title is None) or (compiler is None):
        print('title or compiler not found')
        sys.exit(1)

    if compiler not in COMPILERS:
        print('compiler {} not supported'.format(compiler))
        sys.exit(1)

    build_opts = build.findall('Option')
    
    build_output = None
    for b in build_opts:
        build_output = b.get('output', build_output)
    
    if build_output is None:
        print('output not found')
        sys.exit(1)

    cbp_def_opts = []
    cbp_inc_dirs = []
    for c in build.findall('Compiler/Add'):
        opt = c.get('option')
        inc = c.get('directory')
        if opt:
            if opt.startswith('-D'):
                cbp_def_opts.append(opt[2:])
        if inc:
            cbp_inc_dirs.append(inc)
    cbp_inc_dirs = convert_path(cbp_inc_dirs)
    cbp_inc_dirs = list(map(lambda x: '${workspaceFolder}/' + map_to_root_path(args.root, args.cbp, x),
                            cbp_inc_dirs))

    comp = COMPILERS[compiler]

    comp_cc_opts = comp['cc_opts']
    comp_inc_dirs = list(map(remove_tailing_slash, comp['inc_dirs']))
    comp_def_opts = comp.get('defs', [])

    vscoderoot = os.path.join(args.root, '.vscode')
    if not os.path.exists(vscoderoot):
        os.makedirs(vscoderoot)

    confpath = os.path.join(vscoderoot, 'c_cpp_properties.json')
    outconf = {}
    if os.path.exists(confpath):
        with open(confpath) as fin:
            outconf = json.load(fin)

    if 'version' not in outconf:
        outconf['version'] = 4

    if 'configurations' not in outconf:
        outconf['configurations'] = []

    curconf = {}
    curconf['name'] = title
    curconf['defines'] = cbp_def_opts + comp_def_opts
    curconf['includePath'] = cbp_inc_dirs + comp_inc_dirs
    curconf['compilerPath'] = comp['cc']
    curconf['cStandard'] = 'c99'
    curconf['cppStandard'] = 'c++11'
    curconf['intelliSenseMode'] = 'clang-x86'

    newconfs = []
    for c in outconf['configurations']:
        if c['name'] != title:
            newconfs.append(c)
    
    newconfs.append(curconf)

    outconf['configurations'] = newconfs

    with open(confpath, 'w') as fout:
        fout.write(json.dumps(outconf, indent=4, sort_keys=True))
